#include <Stepper.h>                //Bibliothek der Stepper-Motoren
#include <MySoftwareSerial.h>       //Bibliothek der Servos
#include <DynamixelSoftSerial.h>    //Bibliothek der Servos

///////////////////////////////////////////////////////////////
/////////////GLOBALE VARIABLEN UND KONSTANTEN//////////////////
///////////////////////////////////////////////////////////////

const int stepsPerRevolution = 3200;  //Anzahl an Steps pro Umdrehung
const double stepsPerCM = 29;  //Anzahl an Steps pro cm
const int LTSensor = 10; //Line-Tracking Sensor
const int hubMagnet = 11; //Hubmagnet
const int elektroMagnet = 12; //Elektromagnet
const int dirPinRechts = 6; //dir-Pin für den rechten Stepper
const int stepPinRechts = 7;  //step-Pin für den rechten Stepper
const int dirPinLinks = 8;  //dir-Pin für den linken Stepper
const int stepPinLinks = 9; //step-Pin für den linken Stepper
const int IRLinks = A0;
const int IRRechts = A1;

// initialisierung der Stepper, anhand der Bibliothek
Stepper myStepperLinks(stepsPerRevolution, dirPinLinks, stepPinLinks);
Stepper myStepperRechts(stepsPerRevolution,dirPinRechts,stepPinRechts);

///////////////////////////////////////////////////////////////
/////////////////////////////SETUP/////////////////////////////
///////////////////////////////////////////////////////////////

void setup() {
  pinMode(hubMagnet,OUTPUT);
  digitalWrite(hubMagnet,LOW);
  pinMode(elektroMagnet,OUTPUT);
  digitalWrite(elektroMagnet,LOW);
  myStepperRechts.setSpeed(40);
  myStepperLinks.setSpeed(40);
  pinMode (LTSensor,INPUT);
  pinMode(IRRechts,INPUT);
  pinMode(IRLinks,INPUT);
  Dynamixel.begin(19200, 2, 3, 4);
  Serial.begin(9600);

}


///////////////////////////////////////////////////////////////
/////////////////////////HAUPTCODE/////////////////////////////
///////////////////////////////////////////////////////////////

void loop() {
  
  boolean LTSWert = digitalRead(LTSensor);
   if(analogRead(IRLinks) > 300 && analogRead(IRRechts) > 300){
    forward(1);
   }
   if(analogRead(IRLinks) < 300){
    softTurnRight();
   }
   if(analogRead(IRLinks) < 300){
    softTurnLeft();
   }
   if(analogRead(IRLinks) < 300 && analogRead(IRRechts) < 300){
    backward(8);
    softTurnRight();
   }
   if(LTSWert == 1){
    backward(8);
    pickUp();
    delay(10000);
   }
  }

///////////////////////////////////////////////////////////////
/////////////////////////////METHODEN//////////////////////////
///////////////////////////////////////////////////////////////

//dreht den Roboter um 180° nach links
void turnLeft(double steps){
  backward(8);
  delay(500);
  myStepperRechts.step(steps);
  
}
//dreht den Roboter um 180° nach links
void turnRight(double steps){
  backward(8);
  //drehen
  delay(1000);
  myStepperLinks.step(steps);
}
//bewegt sich um ... cm nach vorne
void forward(double cm){
  for( int i = 0; i<cm;i++){
    myStepperRechts.step(stepsPerCM);
    myStepperLinks.step(stepsPerCM);
  }
}
//bewegt sich um ... cm nach hinten
void backward(double cm){
 for( int i = 0; i<cm;i++){
   myStepperRechts.step(-1*(stepsPerCM));
   myStepperLinks.step(-1*(stepsPerCM));
 }
}
void softTurnLeft(){
  backward(4);
  delay(1000);
  myStepperRechts.step(200);
}
void softTurnRight(){
  backward(4);
  delay(1000);
  myStepperLinks.step(200);
}
void pickUp(){
  //servos runter
  Dynamixel.moveSpeed(1,200, 200);
  Dynamixel.moveSpeed(3,810, 200); 
  delay(2000);
  
  digitalWrite(elektroMagnet,HIGH);
  
  //Ausgngsposition
  Dynamixel.moveSpeed(3,510, 200);
  Dynamixel.moveSpeed(1,810, 200);
  delay(2000);
  
  //in Behälter legen
  /*Dynamixel.moveSpeed(3,450, 200);    //TODO
  delay(2000);
  */
  digitalWrite(elektroMagnet,LOW);
  delay(500);

  //Srvos in Ausgangsposition
  Dynamixel.moveSpeed(1,810, 200);
  Dynamixel.moveSpeed(3,510, 200);
  delay(10000);//zum Testen
}
